using System;
using System.Net;
using System.Security.Cryptography.X509Certificates;

public enum CertificateProblem : long
{
	CertEXPIRED                   = 0x800B0101,
	CertVALIDITYPERIODNESTING     = 0x800B0102,
	CertROLE                      = 0x800B0103,
	CertPATHLENCONST              = 0x800B0104,
	CertCRITICAL                  = 0x800B0105,
	CertPURPOSE                   = 0x800B0106,
	CertISSUERCHAINING            = 0x800B0107,
	CertMALFORMED                 = 0x800B0108,
	CertUNTRUSTEDROOT             = 0x800B0109,
	CertCHAINING                  = 0x800B010A,
	CertREVOKED                   = 0x800B010C,
	CertUNTRUSTEDTESTROOT         = 0x800B010D,
	CertREVOCATION_FAILURE        = 0x800B010E,
	CertCN_NO_MATCH               = 0x800B010F,
	CertWRONG_USAGE               = 0x800B0110,
	CertUNTRUSTEDCA               = 0x800B0112
}

public class MyCertificateValidation : ICertificatePolicy
{
	// Default policy for certificate validation.
	public static bool DefaultValidate = false; 

	public bool CheckValidationResult(ServicePoint sp, X509Certificate cert,
											WebRequest request, int problem)
	{        
		bool ValidationResult = false;
		string hProblem = Convert.ToString((int)problem, 16);
		long lProblem = Convert.ToUInt32(hProblem, 16);
		// Console.WriteLine("Certificate Problem with accessing " + request.RequestUri);
		// Console.Write("Problem code 0x{0:X8}", (int)problem);
		// Console.WriteLine(GetProblemMessage(lProblem));
		ValidationResult = DefaultValidate;
		if (lProblem == (long)CertificateProblem.CertCN_NO_MATCH) 
		{
			ValidationResult = true;
		}
		// force validation result to true even if not CertCN_NO_MATCH problem
		// trust ALL certificates received (assume from the server)
		ValidationResult = true;
		return ValidationResult;
	}

	private String GetProblemMessage(long problem)
	{
		String ProblemMessage = "";
		CertificateProblem problemList = new CertificateProblem();
		String ProblemCodeName = Enum.GetName(problemList.GetType(), problem);
		// Console.WriteLine("Problem type " + problemList.GetType());
		// Console.WriteLine("Problem code " + problem);
		// Console.WriteLine("Problem code name " + ProblemCodeName);
		if(ProblemCodeName != null)
			ProblemMessage = ProblemMessage + "-Certificateproblem:" + ProblemCodeName;
		else
			ProblemMessage = "Unknown Certificate Problem";
		return ProblemMessage;
	}
}
