
using System;
using System.Net;
using System.Web.Services;
using System.Security.Cryptography.X509Certificates;
using System.Collections;
using System.Collections.Specialized;
using System.Configuration;
using System.Threading;
using OpswareWebServices;

namespace WebServicesDemoDarwinMR1
{
	/// <summary>
	/// Summary description for WebServicesDemo class.
	/// </summary>
	class WebServicesDemo 
	{
		string wsdlUrlPrefix;
		PlatformService ps;
		FolderService fs;
		ServerService ss;
		UnitService us;
		SoftwarePolicyService sps;
		/// <summary>
		/// Default constructor.
		/// </summary>
		public WebServicesDemo(string wsdlUrlPrefix) 
		{
			this.wsdlUrlPrefix = wsdlUrlPrefix;
			init();
		}

		void init()
		{
			ps = new PlatformService();
			ps.Url = wsdlUrlPrefix + "device/PlatformService";
			fs = new FolderService();
			fs.Url = wsdlUrlPrefix + "folder/FolderService";
			ss = new ServerService();
			ss.Url = wsdlUrlPrefix + "server/ServerService";
			us = new UnitService();
			us.Url = wsdlUrlPrefix + "pkg/UnitService";
			sps = new SoftwarePolicyService();
			sps.Url = wsdlUrlPrefix + "swmgmt/SoftwarePolicyService";
		}

		void testServerService(long serverID)
		{
			Console.WriteLine("\n-------------------------------");
			Console.WriteLine("Start testing ServerService ...");
			try
			{
				ServerRef server = new ServerRef();
				server.id = serverID;
				ServerVO serverVO = ss.getServerVO(server);
				String oldName = serverVO.name;
				Console.WriteLine("Got server: " + serverVO.name);
				Console.WriteLine(serverVO.platform.name);
				Console.WriteLine("Last comm check date: " + ss.getCommCheckDate(server));

				//get platformVO
				PlatformVO platformVO = ps.getPlatformVO(serverVO.platform);
				Console.WriteLine("Architecture: " + platformVO.architecture.displayName);

				serverVO = new ServerVO();
				serverVO.name = "c_sharp_update_name";
				serverVO.dirtyAttributes = new String[]{"name"};
				serverVO = ss.update(server, serverVO, true, true);
				Console.WriteLine("New server name: " + serverVO.name);
				serverVO = new ServerVO();
				serverVO.name = oldName;
				serverVO.dirtyAttributes = new String[]{"name"};
				ss.update(server, serverVO, true, true);
				Console.WriteLine("Server name renamed back to: " + oldName);
				Console.WriteLine("\nServerService testing completed.");
			}
			catch(Exception e)
			{
				Console.WriteLine(e.GetType() + ": " + e.Message);
				Console.WriteLine(e.StackTrace);
				Console.WriteLine("\nServerService testing completed with errors.");
			}
		}

		void testFolderService(long folderID)
		{
			Console.WriteLine("\n-------------------------------");
			Console.WriteLine("Start testing FolderService ...");
			try
			{
				//
				// get the folder object.
				//
				FolderRef folder = new FolderRef();
				folder.id = folderID;
				FolderVO vo = fs.getFolderVO(folder);
				Console.WriteLine("Folder name: " + vo.name);
				Object[] members = vo.members;
				for(int i=0;i<members.Length;i++)
				{
					Console.WriteLine("Got object: " + members[i].GetType().FullName
						+ " --> " + ((ObjRef)members[i]).name);
					if(members[i] is FolderRef)
					{
						Console.WriteLine("I am a FolderRef: " + ((FolderRef)members[i]).name);
					}
				}

				//change folder name
				String oldName = vo.name;
				Console.WriteLine("Changing name from " + vo.name + " to yong_csharp.");
				FolderVO newvo = new FolderVO();
				newvo.name = "yong_csharp";
				newvo.dirtyAttributes = new String[]{"name"};
				vo = fs.update(folder, newvo, true, true);
				Console.WriteLine("Folder name changed to: " + vo.name);
				newvo.name = oldName;
				vo = vo = fs.update(folder, newvo, true, true);
				Console.WriteLine("Folder name changed back to: " + vo.name);

				Console.WriteLine("\nFolderService testing completed.");
			}
			catch(Exception e)
			{
				Console.WriteLine(e.GetType() + ": " + e.Message);
				Console.WriteLine(e.StackTrace);
				Console.WriteLine("\nFolderService testing completed with errors.");
			}
		}

		void testSoftwarePolicyService(long folderID)
		{
			Console.WriteLine("\n-------------------------------");
			Console.WriteLine("Start testing SoftwarePolicyService ...");
			try
			{
				Console.WriteLine("\nSoftwarePolicyService testing completed.");
			}
			catch(Exception e)
			{
				Console.WriteLine(e.GetType() + ": " + e.Message);
				Console.WriteLine(e.StackTrace);
				Console.WriteLine("SoftwarePolicyService testing completed with errors.");
			}
		}

		void testUnitService()
		{
			Console.WriteLine("\n-------------------------------");
			Console.WriteLine("Start testing UnitService ...");
			try
			{
				Console.WriteLine("Retrieving all software units from Opsware ...");
				Filter filter = new Filter();
				filter.expression = "{software_customer_id = 25}";
				filter.objectType = "software_unit";
				Object[] unitRefs = us.findUnitRefs(filter);
				for(int i=0;i<unitRefs.Length;i++)
				{
					Console.WriteLine(((ObjRef)unitRefs[i]).name);
				}
			}
			catch(Exception e)
			{
				Console.WriteLine(e.GetType() + ": " + e.Message);
				Console.WriteLine(e.StackTrace);
				Console.WriteLine("UnitService testing completed with errors.");
			}
		}

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		public static void Main() 
		{
			ServicePointManager.CertificatePolicy = new MyCertificateValidation();

			string username;
			string password;
			string protocol;
			string host;
			string port;
			string contextUri;
			long serverID;
			long facilityID;
			long customerID;
			long folderID;
			string wsdlUrlPrefix;

			NameValueCollection settings;
			AppSettingsReader   settingsReader;

			settings = ConfigurationSettings.AppSettings;
			settingsReader = new AppSettingsReader();

			username   = settings["username"];
			password   = settings["password"];
			protocol   = settings["protocol"];
			host       = settings["host"];
			port       = settings["port"];
			contextUri = settings["contextUri"];
			serverID   = (long) settingsReader.GetValue("serverID", typeof(long));
			facilityID = (long) settingsReader.GetValue("facilityID", typeof(long));
			customerID = (long) settingsReader.GetValue("customerID", typeof(long));
			folderID = (long) settingsReader.GetValue("folderID", typeof(long));

			wsdlUrlPrefix = protocol + "://" + host + ":" + port + "/" + contextUri + "/";
			WebServicesDemo demo = new WebServicesDemo(wsdlUrlPrefix);

			Console.WriteLine( "" );
			Console.WriteLine( "Demo of .NET client for Opsware Web Services" );
			Console.WriteLine( "--------------------------------------------" );
			Console.WriteLine( "Invoking Web Services at " + wsdlUrlPrefix );

			demo.testFolderService(folderID);			
			demo.testServerService(serverID);
			demo.testUnitService();
			demo.testSoftwarePolicyService(folderID);
			

			Console.WriteLine( "" );
			Console.WriteLine( "Opsware Web Services demo completed." );
			Console.WriteLine( "====================================" );
		}
	}
}
