#!/usr/bin/perl -w

# initialize SOAP::Lite proxy module
use SOAP::Lite +autodispatch =>
    on_fault => sub { my ($soap, $res) = @_; 
        die ref $res ? $res->faultdetail : $soap->transport->status, "\n";
    },
    #trace => 'debug' # uncomment to trace SOAP messages
;
use Data::Dumper;
#$ENV{HTTPS_CA_FILE} = "path_to/opsware-ca.crt";

print "Initializing web service ports...\n";
my $username = "integration";
my $password = "integration";
my $protocol = "https";
my $host = "occ.silversox.qa.opsware.com";
#my $host = "localhost";
my $port = "443";
my $contextUri = "osapi/com/opsware/";

# create the serverService port
my $serverServiceName = "server/ServerService";
my $serverUri   = "http://www.opsware.com/" . $contextUri .$serverServiceName;
my $serverProxy = $protocol . "://" . $username . ":" . $password . "@" . $host 
		. ":" . $port . "/" . $contextUri . $serverServiceName;
my $serverPort = SOAP::Lite
	-> uri($serverUri)
	-> proxy($serverProxy);

# create the FolderService port
my $folderServiceName = "folder/FolderService";
my $folderUri   = "http://www.opsware.com/" . $contextUri . $folderServiceName;
my $folderProxy = $protocol . "://" . $username . ":" . $password . "@" . $host 
		. ":" . $port . "/" . $contextUri . $folderServiceName;
my $folderPort = SOAP::Lite
    -> uri($folderUri)
    -> proxy($folderProxy);

#create the swPolicyService port
my $swPolicyServiceName = "swmgmt/SoftwarePolicyService";
my $swPolicyProxy = $protocol . "://" . $username . ":" . $password . "@" . $host 
		. ":" . $port . "/" . $contextUri . $swPolicyServiceName;
my $swPolicyUri = "http://www.opsware.com/" . $contextUri . $swPolicyServiceName;
my $swPolicyPort = SOAP::Lite
	-> uri($swPolicyUri)
	-> proxy($swPolicyProxy);

# create the unitService port
my $unitServiceName = 'pkg/UnitService';
my $unitUri = "http://www.opsware.com/" . $contextUri . $unitServiceName;
my $unitProxy = $protocol . "://" . $username . ":" . $password . "@" . $host 
		. ":" . $port . "/" . $contextUri . $unitServiceName;
my $unitPort = SOAP::Lite
	-> uri($unitUri)
	-> proxy($unitProxy);	
	
# use this subroutine to get an exceptionName out of a SOAP::SOM object
sub getExceptionName {
    my $fault = shift; #get the fault object
    if($fault->faultdetail->{'fault'}){
        return ref($fault->faultdetail->{'fault'});
    }
}

print "Calling getRoot on folderport with no parameters...\n";
my $root = $folderPort->getRoot()->result();
print ' Got root folder: ' . $root->{'name'} . "\n";


my $rootVO = $folderPort->getFolderVO(SOAP::Data->name('self')
				->value(\SOAP::Data->name('id')->type('long')->value($root->{'id'})))->result();

my $folder = SOAP::Data->name('self')
				->attr({ 'xmlns:ns_fs' => 'http://folder.opsware.com'})
				->type('ns_fs:FolderRef')->value(\SOAP::Data->name('id')->type('long')
				->value($root->{'id'}));

print "Calling getChildren on folderport with a folder Ref...\n";
my $children = $folderPort->getChildren($folder, 
			SOAP::Data->name('type')->type('string')->value(''))->result();
foreach $child (@$children){
	print 'Get child: ' . $child->{'name'} . "\n";
}


my $serverID = 40038;
my $server = SOAP::Data->name('self')->value(\SOAP::Data->name('id')->type('long')->value($serverID));
print "Calling getSErverVO on server port...\n";
my $firstVO = $serverPort->SOAP::getServerVO($server);
print 'Got server: ', $firstVO->{'name'}, '(', $firstVO->{'description'}, ")\n";

my @dirtyAttrs = ('description');
my $serverVO = SOAP::Data->name('vo')
		->attr({ 'xmlns:ns_ss' => 'http://server.opsware.com'})
		->value(\SOAP::Data->value(
		SOAP::Data->name('description')->value('PERL_UPDATE_DESC')->type('string'),
		SOAP::Data->name('logChange')->value('false')->type('boolean'),
		SOAP::Data->name('dirtyAttributes' =>
						   \SOAP::Data->name("element" => @dirtyAttrs)
							      ->type("string"))
					  ->type("ns_ss:ArrayOf_soapenc_string"),
        ));
my $force = SOAP::Data->name('force')->value('true')->type('boolean');
my $refetch = SOAP::Data->name('refetch')->value('true')->type('boolean');

#call the update method
print 'Calling method serverWSPort.update on serverPort...', "\n";
my $updatedServerVO = $serverPort->update(
                         $server,
                         $serverVO,
                         $force,
                         $refetch)->result();
print "New description: ", $updatedServerVO->{'description'}, "\n";         


# remove a folder --> works
#my $folderToBeRemoved = SOAP::Data->name('self')
#			->attr({ 'xmlns:ns_fs' => 'http://folder.opsware.com'})
#			->type('ns_fs:FolderRef')->value(\SOAP::Data->name('id')->type('long')
#				->value(17420040));
#$folderPort->remove($folderToBeRemoved);

# test remove policy --> works
#my $swpId = 2260038;
#my $swpolicy = SOAP::Data->name('self')
#			->attr({ 'xmlns:ns_swps' => 'http://swmgmt.opsware.com'})
#			->type('ns_swps:SoftwarePolicyRef')->value(\SOAP::Data->name('id')->type('long')
#				->value($swpId));
#$swPolicyPort->remove($swpolicy);

print "Calling findServerRefs on a null value...\n";
my $myRefs = $serverPort->SOAP::findServerRefs(); # passing a null value returns all accessible refs
#print Dumper($myRefs);

print "Constructing Soap::Data server Refs...\n";
my @refs = [];
foreach my $ref (@$myRefs){ 
     my $object = SOAP::Data->name('ServerRef')
                      ->value(\SOAP::Data->name('id')
                              ->type('long')
                              ->value($ref->{'id'}
                       )
                  )->attr({ 'xmlns:server' => 'http://server.opsware.com'})
                   ->type('server:ServerRef');
     push @refs, $object;
}

print "Constructing a Soap::Data array...\n";
my $selves = SOAP::Data->name("selves" => 
                 \SOAP::Data->name("element" => @refs)->type("server:ServerRef"))
             ->attr({ 'xmlns:server' => 'http://server.opsware.com'})
             ->type("server:ArrayOfServerRef");

print "Calling getServerVOs on the array...\n";
my $returned = $serverPort->getServerVOs($selves);

print "Checking if there is an error...\n";
if($returned->fault){
    print $returned->faultstring . "\n"; #this will print the error msg
    print "ExceptionName: " . getExceptionName($returned) . "\n"; # a NotFoundException should be displayed here
    exit(1);

}
print "getting the result...\n";
my $serverVOs = $returned->result;

print "Dumping the first VO...\n";
if($#$serverVOs > 0){
    print Dumper($$serverVOs[0]);
}
else{
    print "Did not return any Server VOs\n"
}

my $filter = SOAP::Data->name('filter')
		->value(\SOAP::Data->value(
		    SOAP::Data->name('expression')->value('(software_name *=* "ismto") & (software_platform_name  *=* "sunos 5.10 x86")')->type('string'),
                    SOAP::Data->name('objectType')->value("software_unit")->type('string')
                    )
                );
print "Calling findUnitRefs on a unitPort for all sun, ismtool units...\n";
$unitRefs = $unitPort->findUnitRefs($filter)->result;
my @types = ();
for my $unitRef (@$unitRefs){
    my $type = ref $unitRef;
    my $found = 0;
    for my $aType (@types){
    	if($aType eq $type){
    	    $found = 1;
    	    last;
    	}
    }
    if(!$found){
    	print $type . ".\n";
    	push @types, $type;
    }    
}

my $found = $#types + 1;
print "found \'" . $found . "\' different types:\t";
for my $type (@types){
    print $type . ", ";
}
print "\n";

print Dumper($$unitRefs[0]);

print "Constructing each different ref\n";
@refs = ();
for my $unitRef (@$unitRefs){
    my $type = ref $unitRef;
    my $object = SOAP::Data->name($type)
                          ->value(\SOAP::Data->name('id')
                                  ->type('long')
                                  ->value($unitRef->{'id'}
                           )
                      )->attr({ 'xmlns:solpkg' => 'http://solaris.pkg.opsware.com'})
                       ->type("solpkg:$type");
     push @refs, $object;
}

print "Constructing a Soap::Data array...\n";
$selves = SOAP::Data->name("selves" => 
                 \SOAP::Data->name("element" => @refs))
             ->attr({ 'xmlns:unit' => 'http://pkg.opsware.com'})
             ->type("unit:ArrayOf_xsd_anyType");

print "Calling getUnitVOs on the array...\n";
my $unitVOs = $unitPort->getUnitVOs($selves)->result;

print "Dumping the first VO...\n";
if($#$unitVOs > 0){
    print Dumper($$unitVOs[0]);
}
else{
    print "Did not return any unit VOs\n"
}
